import { marked } from 'marked';
import type { Post } from '@blog/models/Post';
import type { PostMeta } from '@blog/models/PostMeta';
import * as yamlFront from 'yaml-front-matter';
import { readdir } from 'node:fs/promises';

export async function openPostMarkdownFile(filename: string): Promise<string> {
  const file = Bun.file(filename);
  return file.text();
}

type PostMetaWithRawContent = {
  meta: PostMeta,
  content: string
};

export function parsePostMetadata(post: string): PostMetaWithRawContent {
  const parsedData = yamlFront.loadFront(post);

  return {
    meta: {
      title: parsedData.title,
      description: parsedData.description,
      date: new Date(parsedData.date),
      draft: parsedData?.draft || false,
      tags: parsedData.tags,
    },
    content: parsedData.__content
  };
}

export async function getPostList(path: string): Promise<PostMeta[]> {
  const postList: PostMeta[] = new Array<PostMeta>();
  const dir = await readdir(path);

  if (!dir.length) {
    return postList;
  }

  for (const file of dir) {
    const fileContent = await openPostMarkdownFile(path + '/' + file);
    const postMeta = parsePostMetadata(fileContent);
    postMeta.meta.slug = "/posts/" + file.slice(0, -3);
    postList.push(postMeta.meta);
  }

  return postList;
}

export async function readPostMarkdown(filename: string): Promise<Post> {
  const contents = await openPostMarkdownFile(filename);
  const post = parsePostMetadata(contents);
  post.meta.slug = "/posts/" + filename.slice(0, -3);
  const parsedPost = await marked.parse(post.content);

  return {
    meta: post.meta,
    content: post.content,
    html: parsedPost
  };
}

